;;;*********************************************
;;;*               REAL                        *
;;;*                    /                      *
;;;*                       FRAC                *
;;;*               Realfrac.lsp                *
;;;*        written by: Allen Proffit          *
;;;*        Dependable Design Drafting         *
;;;*  Copyright (c) 1994 Robert A. Proffit     *
;;;*       All rights reserved by author       *
;;;*                                           *
;;;*********************************************                               
;;;    
;;;
;;;RFDIM
;;;Provides commands (in CAPS) at AutoCAD command line
;;;
(defun c:rfdim () 
  (prompt 
"\nDim Algn., Dim Horiz., Dim DIAmeter, Dim Rot., Dim RADius, 
  Dim Vert. or Chg Frac"
  )                                  ;prompt
  (princ)
) 
;;;rfdim
;;;
;;;CF (Change Fraction)
;;;Changes one existing fraction to the Realfrac format
;;;Input:  one dimension entity picked by user
;;;Output: deletes existing dimension, adds new Realfrac dimsnsion
;;;
(defun c:cf (/ pt1 pt2 pt3 dimstr pipe enl iffer curlay isit assocx flag 
               startild dimshow counter holdlist chechar onechar entit enti ent 
               pt4 pt5 pt6 pp numchar ang rotangr rotangd dimt
            ) 
  (setq origerr *error*)
  (setq *error* cferr)
  (setvar "cmdecho" 0)
  (setq curlay (getvar "clayer"))
  (setq assocx (getvar "dimaso"))
  (setq entit (entget (car (entsel))))
  (setq enti (car entit))
  (setq ent (cdr enti))
  (if (and (= (cdr (assoc 0 entit)) "DIMENSION") 
        (or (= (cdr (assoc 70 entit)) 0) (= (cdr (assoc 70 entit)) 1) 
            (= (cdr (assoc 70 entit)) 3) 
            (= (cdr (assoc 70 entit)) 4)
        )                            ;or
      )                              ;and
    (entdel ent) 
    (command ^c)
  )                                  ;if
  (diminfo) 
  (presuf) 
  (gedimtxt) 
  (addprfix) 
  (setvar "dimaso" assocx)
  (command "layer" "s" curlay "") 
  (if (= flag "frac") 
    (command2) 
    (command1)
  )                                  ;if
  (setvar "cmdecho" 1)
  (setq *error* origerr)
  (princ)
) 
;;;cf  
;;;
;;;DA (Dimension Aligned)
;;;Creates one new aligned dimension with Realfrac text format
;;;Input:  points supplied by user to create dimension
;;;Output: one new dimension with Realfrac formatted text
;;;
(defun c:da (/ pt1 pt2 pt3 dimstr pipe enl iffer curlay isit assocx flag 
               startild dimshow counter holdlist chechar onechar entit enti ent 
               pt4 pt5 pt6 pp numchar ang rotangr rotangd dimt pta ptb ptc
            ) 
  (setq origerr *error*)
  (setq *error* cferr)
  (setq pta (getpoint "\nFirst extension line origin: ") 
        ptb (getpoint "\nSecond extension line origin: ") 
        ptc (getpoint "\nDimension line location: ")) ;setq
  (command "dim" "aligned" pta ptb ptc "" ^c) 
  (setvar "cmdecho" 0)
  (setq entit (entget (entlast)))
  (setq curlay (getvar "clayer"))
  (setq assocx (getvar "dimaso"))
  (setq enti (car entit))
  (setq ent (cdr enti))
  (entdel ent) 
  (diminfo) 
  (presuf) 
  (gedimtxt) 
  (addprfix) 
  (setvar "dimaso" assocx)
  (command "layer" "s" curlay "") 
  (if (= flag "frac") 
    (command2) 
    (command1)
  )                                  ;if
  (setvar "cmdecho" 1)
  (setq *error* origerr)
  (princ)
) 
;;;da  
;;;
;;;DDIA (Dimension DIAmeter)
;;;Creates one new diameter dimension with Realfrac text format
;;;Input:  points and arc by user to create dimension
;;;Output: one new dimension with Realfrac formatted text
;;;
(defun c:ddia (/ pt1 pt2 pt3 dimstr pipe enl iffer curlay isit assocx flag 
                 startild dimshow counter holdlist chechar onechar entit enti 
                 ent pt4 pt5 pt6 pp numchar ang rotangr rotangd dimt pta ptb 
                 ptc
              ) 
  (setq origerr *error*)
  (setq *error* cferr)
  (setq dimcirc (car (cdr (entsel "Pick circle or arc: "))))
  (setq ldrdist (getdist "Leader distance: "))
  (command "dim" "diameter" dimcirc "" ldrdist ^c) 
  (setvar "cmdecho" 0)
  (setq entit (entget (entlast)))
  (setq curlay (getvar "clayer"))
  (setq assocx (getvar "dimaso"))
  (setq enti (car entit))
  (setq ent (cdr enti))
  (entdel ent) 
  (diminfo) 
  (presuf) 
  (gedimtxt) 
  (addprfix) 
  (setvar "dimaso" assocx)
  (command "layer" "s" curlay "") 
  (if (= flag "frac") 
    (command2) 
    (command1)
  )                                  ;if
  (setvar "cmdecho" 1)
  (setq *error* origerr)
  (princ)
) 
;;;ddia  
;;;
;;;DA (Dimension Horizontal)
;;;Creates one new horizontal dimension with Realfrac text format
;;;Input:  points supplied by user to create dimension
;;;Output: one new dimension with Realfrac formatted text
;;;
(defun c:dh (/ pt1 pt2 pt3 dimstr pipe enl iffer curlay isit assocx flag 
               startild dimshow counter holdlist chechar onechar entit enti ent 
               pt4 pt5 pt6 pp numchar ang rotangr rotangd dimt pta ptb ptc
            ) 
  (setq origerr *error*)
  (setq *error* cferr)
  (setq pta (getpoint "\nFirst extension line origin: ") 
        ptb (getpoint "\nSecond extension line origin: ") 
        ptc (getpoint "\nDimension line location: ")) ;setq
  (command "dim" "horizontal" pta ptb ptc "" ^c) 
  (setvar "cmdecho" 0)
  (setq entit (entget (entlast)))
  (setq curlay (getvar "clayer"))
  (setq assocx (getvar "dimaso"))
  (setq enti (car entit))
  (setq ent (cdr enti))
  (entdel ent) 
  (diminfo) 
  (presuf) 
  (gedimtxt) 
  (addprfix) 
  (setvar "dimaso" assocx)
  (command "layer" "s" curlay "") 
  (if (= flag "frac") 
    (command2) 
    (command1)
  )                                  ;if
  (setvar "cmdecho" 1)
  (setq *error* origerr)
  (princ)
) 
;;;dh  
;;;
;;;DR (Dimension Rotated)
;;;Creates one new rotated dimension with Realfrac text format
;;;Input:  points and angle supplied by user to create dimension
;;;Output: one new dimension with Realfrac formatted text
;;;
(defun c:dr (/ pt1 pt2 pt3 dimstr pipe enl iffer curlay isit assocx flag 
               startild dimshow counter holdlist chechar onechar entit enti ent 
               pt4 pt5 pt6 pp numchar ang rotangr rotangd dimt pta ptb ptc
            ) 
  (setq origerr *error*)
  (setq *error* cferr)
  (setq ang (getangle "\nDimension line angle: ") 
        pta (getpoint "\nFirst extension line origin: ") 
        ptb (getpoint "\nSecond extension line origin: ") 
        ptc (getpoint "\nDimension line location: ") 
        rotang (/ (* ang 180) pi))   ;setq
  (command "dim" "rot" rotang pta ptb ptc "" ^c) 
  (setvar "cmdecho" 0)
  (setq entit (entget (entlast)))
  (setq curlay (getvar "clayer"))
  (setq assocx (getvar "dimaso"))
  (setq enti (car entit))
  (setq ent (cdr enti))
  (entdel ent) 
  (diminfo) 
  (presuf) 
  (gedimtxt) 
  (addprfix) 
  (setvar "dimaso" assocx)
  (command "layer" "s" curlay "") 
  (if (= flag "frac") 
    (command2) 
    (command1)
  )                                  ;if
  (setvar "cmdecho" 1)
  (setq *error* origerr)
  (princ)
) 
;;;dr  
;;;
;;;DRAD (Dimension RADius)
;;;Creates one new radius dimension with Realfrac text format
;;;Input:  points and arc by user to create dimension
;;;Output: one new dimension with Realfrac formatted text
;;;
(defun c:drad (/ pt1 pt2 pt3 dimstr pipe enl iffer curlay isit assocx flag 
                 startild dimshow counter holdlist chechar onechar entit enti 
                 ent pt4 pt5 pt6 pp numchar ang rotangr rotangd dimt pta ptb 
                 ptc
              ) 
  (setq origerr *error*)
  (setq *error* cferr)
  (setq dimcirc (car (cdr (entsel "Pick circle or arc: "))))
  (setq ldrdist (getdist "Leader distance: "))
  (command "dim" "radius" dimcirc "" ldrdist ^c) 
  (setvar "cmdecho" 0)
  (setq entit (entget (entlast)))
  (setq curlay (getvar "clayer"))
  (setq assocx (getvar "dimaso"))
  (setq enti (car entit))
  (setq ent (cdr enti))
  (entdel ent) 
  (diminfo) 
  (presuf) 
  (gedimtxt) 
  (addprfix) 
  (setvar "dimaso" assocx)
  (command "layer" "s" curlay "") 
  (if (= flag "frac") 
    (command2) 
    (command1)
  )                                  ;if
  (setvar "cmdecho" 1)
  (setq *error* origerr)
  (princ)
) 
;;;drad  
;;;
;;;DV (Dimension Vertical)
;;;Creates one new vertical dimension with Realfrac text format
;;;Input:  points supplied by user to create dimension
;;;Output: one new dimension with Realfrac formatted text
;;;
(defun c:dv (/ pt1 pt2 pt3 dimstr pipe enl iffer curlay isit assocx flag 
               startild dimshow counter holdlist chechar onechar entit enti ent 
               pt4 pt5 pt6 pp numchar ang rotangr rotangd dimt pta ptb ptc
            ) 
  (setq origerr *error*)
  (setq *error* cferr)
  (setq pta (getpoint "\nFirst extension line origin: ") 
        ptb (getpoint "\nSecond extension line origin: ") 
        ptc (getpoint "\nDimension line location: ")) ;setq
  (command "dim" "vertical" pta ptb ptc "" ^c) 
  (setvar "cmdecho" 0)
  (setq entit (entget (entlast)))
  (setq curlay (getvar "clayer"))
  (setq assocx (getvar "dimaso"))
  (setq enti (car entit))
  (setq ent (cdr enti))
  (entdel ent) 
  (diminfo) 
  (presuf) 
  (gedimtxt) 
  (addprfix) 
  (setvar "dimaso" assocx)
  (command "layer" "s" curlay "") 
  (if (= flag "frac") 
    (command2) 
    (command1)
  )                                  ;if
  (setvar "cmdecho" 1)
  (setq *error* origerr)
  (princ)
) 
;;;dv  
;;;
;;;CB (Change Back)
;;;Converts one dimension to default dimension format
;;;Input:  dimension supplied by user
;;;Output: dimension converted to default text
;;;
(defun c:cb () 
  (setq rdim (car (entsel)))
  (command "dim" "newtext" "" rdim "" ^c) 
  (princ)
) 
;;;cb
;;;
;;;addprfix
;;;adds Realfrac prefixes to one properly formatted fraction
;;;Input:  dimstr (dimension string - fraction in std AutoCAD format)
;;;Output: dimshow (dimension string with Realfrac prefixes added)
;;;
(defun addprfix () 
  (setq numchar (strlen dimstr))
  (setq counter 1)
  (setq holdlist (list ""))
  (setq pipe 1)
  (setq startild 1)
  (setq dimshow "")
  (repeat numchar 
    (setq onechar (list (substr dimstr counter 1)))
    (setq holdlist (append holdlist onechar))
    (setq counter (1+ counter))
  )                                  ;repeat
  (setq counter 1)
  (repeat numchar 
    (if (eq (nth counter holdlist) " ") 
      (setq startild counter)
    ) 
    (setq counter (1+ counter))
  ) 
  (setq counter 1)
  (repeat numchar 
    (if (eq (nth counter holdlist) "/") 
      (setq pipe counter)
    ) 
    (setq counter (1+ counter))
  ) 
  (setq counter 1)
  (repeat (- startild 1) 
    (setq dimshow (strcat dimshow (nth counter holdlist)))
    (setq counter (1+ counter))
  ) 
  (setq counter startild)
  (repeat (- pipe startild) 
    (setq dimshow (strcat dimshow (strcat "~" (nth counter holdlist))))
    (setq counter (1+ counter))
  ) 
  (setq dimshow (strcat dimshow "|" "/"))
  (setq counter (+ 1 pipe))
  (repeat (- numchar (+ 1 pipe)) 
    (setq dimshow (strcat dimshow (strcat "`" (nth counter holdlist))))
    (setq counter (1+ counter))
  ) 
  (setq dimshow (strcat dimshow (chr 34)))
  (princ)
) 
;;;addprfix
;;;
;;;command1
;;;called to provide dimension command (normal)
;;;Input:  dimt (dimension type)
;;;Output: dimension command
;;;
(defun command1 () 
  (if (eq dimt "aligned") 
    (command "dim" "aligned" pt4 pt5 pt1 "" ^c)
  )                                  ;if 
  (if (= dimt "horizontal") 
    (command "dim" "horizontal" pt4 pt5 pt1 "" ^c)
  )                                  ;if 
  (if (= dimt "vertical") 
    (command "dim" "vertical" pt4 pt5 pt1 "" ^c)
  )                                  ;if 
  (if (= dimt "rotated") 
    (command "dim" "rot" rotangd pt4 pt5 pt1 "" ^c)
  )                                  ;if 
  (if (= dimt "diameter") 
    (command "dim" "diameter" pt6 "" (rtos ldrdist) "" ^c)
  )                                  ;if 
  (if (= dimt "radius") 
    (command "dim" "radius" pt6 "" (rtos ldrdist) "" ^c)
  )                                  ;if 
  (princ)
) 
;;;command1  
;;;
;;;diminfo (dimension information)
;;;Input:  entit (dimension entity)
;;;Output: pt1 - pt6 (dimension points)
;;;        dt (AutoCAD dimension type)
;;;        rotangr (rotation angle in radians)
;;;        rotangd (rotation angle in degrees)
;;;        dimt (dimension type - text)
;;;
(defun diminfo () 
  (setq dimt "unknown")
  (setq pt1 (cdr (assoc 10 entit)))
  (setq pt2 (cdr (assoc 11 entit)))
  (setq pt3 (cdr (assoc 12 entit)))
  (setq pt4 (cdr (assoc 13 entit)))
  (setq pt5 (cdr (assoc 14 entit)))
  (setq pt6 (cdr (assoc 15 entit)))
  (setq dt (cdr (assoc 70 entit)))
  (setq rotangr (cdr (assoc 50 entit)))
  (setq rotangd (/ (* rotangr 180) pi))
  (setq ldrdist (cdr (assoc 40 entit)))
  (if (eq dt 1) 
    (setq dimt "aligned")
  )                                  ;if
  (if (eq dt 3) 
    (setq dimt "diameter")
  )                                  ;if
  (if (eq dt 4) 
    (setq dimt "radius")
  )                                  ;if
  (if (and (eq dt 0) 
        (eq rotangr 0)
      ) 
    (setq dimt "horizontal")
  )                                  ;if
  (if (and (eq dt 0) 
        (= rotangd 90)
      ) 
    (setq dimt "vertical")
  )                                  ;if
  (if (and (eq dt 0) 
        (and (/= rotangd 0) 
          (/= rotangd 90)
        )
      ) 
    (setq dimt "rotated")
  )                                  ;if
  (princ)
) 
;;;diminfo
;;;
;;;gedimtxt (get dimension text)
;;;Input:  enti (dimension list)
;;;Output: dimstr (dimension text string)
;;;
(defun gedimtxt () 
  (setq dimstr "")
  (setq entit (entget (cdr enti)))   ;(setq enti (car entit))
  (setq ent (cdr enti))
  (setvar "dimaso" 0)
  (if (= nil (tblsearch "LAYER" "copylayr")) 
    (command "layer" "n" "copylayr" "")
  ) 
  (command "point" "0,0") 
  (setq pp (cdr (car (entget (entlast)))))
  (command "layer" "s" "copylayr" "") 
  (command1) 
  (while (/= iffer "yes") 
    (setq enl (cdr (entget (entlast))))
    (if (cdr (assoc 1 enl)) 
      (setq dimstr (strcat dimstr (cdr (assoc 1 enl))))
    )                                ;if
    (if (/= (cdr (assoc 8 enl)) "COPYLAYR") 
      (setq iffer "yes")
    )                                ;if
    (if (= (cdr (assoc 8 enl)) "COPYLAYR") 
      (entdel (entlast))
    )                                ;if
  )                                  ;while
  (setq numch (strlen dimstr))
  (if (/= (strcat pre suf) "") 
    (setq dimstr (substr dimstr 1 (- numch (+ prelnth suflnth))))
  )                                  ;if
  (setq numchar (strlen dimstr))
  (setq isit dimstr)
  (setq dimmer dimstr)
  (repeat numchar 
    (setq chechar (substr isit 1 1))
    (if (= chechar "/") 
      (setq flag "frac")
    ) 
    (setq isit (substr isit 2))
  )                                  ;repeat
  (entdel pp) 
  (princ)
) 
;;;gedimtxt
;;;
;;;presuf
;;;gets dimpost prefix and suffix
;;;Input:  dimvar "dimpost"
;;;Output: pre (dimpost prefix)
;;;        suf (dimpost suffix)
;;;        prelnth (string length of pre)
;;;        suflnth (string lenght of suf)
;;;
(defun presuf (/ pstr numchar counter chechar) 
  (setq pstr (getvar "dimpost"))
  (setq numchar (strlen pstr))
  (setq counter 1 
        flag1 0 
        flag2 0 
        pre "" suf "" 
        chechar q)                   ;setq
  (repeat numchar 
    (setq chechar (substr pstr counter 1))
    (if (= chechar "<") 
      (setq flag1 counter)
    )                                ;if 
    (setq counter (1+ counter))
  )                                  ;repeat 
  (setq counter 1)
  (repeat numchar 
    (setq chechar (substr pstr counter 1))
    (if (= chechar ">") 
      (setq flag2 counter)
    )                                ;if 
    (setq counter (1+ counter))
  )                                  ;repeat 
  (if (= (- flag2 flag1) 1) 
    (setq pre (substr pstr 1 (- flag1 1)))
  )                                  ;if  
  (if (= (- flag2 flag1) 1) 
    (setq suf (substr pstr (+ flag2 1) numchar))
    (setq suf pstr)
  )                                  ;if
  (setq prelnth (strlen pre))
  (setq suflnth (strlen suf))
  (princ)
) 
;;;presuf
;;;
;;;command2
;;;provides dimension command with Reakfrac formatted text
;;;Input:  dimt (dimension type)
;;;        dimshow (formatted dimension string)
;;;Output: dimension command
;;;
(defun command2 () 
  (setq dimshow (strcat pre dimshow suf))
  (if (eq dimt "aligned") 
    (command "dim" "aligned" pt4 pt5 pt1 dimshow "" ^c)
  )                                  ;if 
  (if (= dimt "horizontal") 
    (command "dim" "horizontal" pt4 pt5 pt1 dimshow "" ^c)
  )                                  ;if 
  (if (= dimt "vertical") 
    (command "dim" "vertical" pt4 pt5 pt1 dimshow "" ^c)
  )                                  ;if 
  (if (= dimt "rotated") 
    (command "dim" "rotated" rotangd pt4 pt5 pt1 dimshow "" ^c)
  )                                  ;if 
  (if (= dimt "diameter") 
    (command "dim" "diameter" pt6 dimshow (rtos ldrdist) "" ^c)
  )                                  ;if 
  (if (= dimt "radius") 
    (command "dim" "radius" pt6 dimshow (rtos ldrdist) "" ^c)
  )                                  ;if 
  (princ)
) 
;;;command2  
;;;
;;;cc
;;;Control Characters
;;;provides control characters in alert box
;;;
(defun c:cc () 
  (setq line1 "\n TEXT POSITION       CONTROL CHAR.      CHAR")
  (setq line2 "\n super / frac numer  tilde (~)          any")
  (setq line3 "\n sub / frac denom    rev apos (`)       any")
  (setq line4 "\n fraction bar        vert bar (|)       slash")
  (setq line5 "\n small space         tilde (`)          space")
  (setq line6 "\n numerator super     open brac ({)      any")
  (setq line7 "\n numerator sub       close brac (})     any")
  (setq line8 "\n denominator super   open sq brac ([)   any")
  (setq line9 "\n denominator subs    close sq brac (])  any")
  (setq line10 "\n bar @ numer frac's  tilde (~)          slash")
  (setq line11 "\n bar @ denom frac's  rev apos (`)       slash")
  (setq alstr1 (strcat line1 line2 line3 line4))
  (setq alstr2 (strcat line5 line6 line7 line8))
  (setq alstr3 (strcat line9 line10 line11))
  (setq alstr (strcat alstr1 alstr2 alstr3))
  (alert alstr) 
  (princ)
) 
;;;cc
;;;
;;;cferr
;;;Change Fraction error
;;;error trapping for Realfrac dimension functions
;;;
(defun cferr (msg) 
  (setvar "dimaso" assocx)
  (command "layer" "s" curlay "") 
  (princ "\nError in CHGFRAC: ") 
  (entmake entit) 
  (princ msg) 
  (setq *error* origerr)
  (setvar "cmdecho" 1)
  (princ)
) 
;;;cferr

